@@ -1,12 +1,13 @@
-import importlib
-import logging
-import os
-import sys
-import traceback
+from importlib import import_module
+from logging import getLogger
+from os import listdir
+from os.path import abspath, dirname, isfile, join as pathjoin
+from sys import exc_info, path as syspath
+from traceback import extract_tb
 
 from grizzly.common import Adapter
 
-log = logging.getLogger(__name__)  # pylint: disable=invalid-name
+LOG = getLogger(__name__)
 
 __all__ = ("get", "load", "names")
 __adapters__ = dict()
@@ -17,29 +18,29 @@ def get(name):
 def load(path=None, skip_failures=True):
     assert not __adapters__, "adapters have already been loaded"
     if path is None:
-        path = os.path.dirname(__file__)
-    path = os.path.abspath(path)
-    log.debug("loading adapters from %r", path)
-    sys.path.append(path)
-    for sub in os.listdir(path):
-        if not os.path.isfile(os.path.join(path, sub, "__init__.py")):
+        path = dirname(__file__)
+    path = abspath(path)
+    LOG.debug("loading adapters from %r", path)
+    syspath.append(path)
+    for sub in listdir(path):
+        if not isfile(pathjoin(path, sub, "__init__.py")):
             continue
-        log.debug("scanning %r", sub)
+        LOG.debug("scanning %r", sub)
         try:
-            lib = importlib.import_module(sub)
+            lib = import_module(sub)
         except Exception:  # pylint: disable=broad-except
             if not skip_failures:
                 raise
-            exc_type, exc_obj, exc_tb = sys.exc_info()
-            tbinfo = traceback.extract_tb(exc_tb)[-1]
-            log.debug("raised %s: %s (%s:%d)", exc_type.__name__, exc_obj, tbinfo[0], tbinfo[1])
+            exc_type, exc_obj, exc_tb = exc_info()
+            tbinfo = extract_tb(exc_tb)[-1]
+            LOG.debug("raised %s: %s (%s:%d)", exc_type.__name__, exc_obj, tbinfo[0], tbinfo[1])
             continue
         for clsname in dir(lib):
             cls = getattr(lib, clsname)
             if isinstance(cls, type) and issubclass(cls, Adapter):
                 if clsname == "Adapter":
                     continue
-                log.debug("sanity checking %r", clsname)
+                LOG.debug("sanity checking %r", clsname)
                 if not isinstance(cls.NAME, str):
                     raise RuntimeError(
                         "%s.NAME must be 'str' not %r" % (cls.__name__, type(cls.NAME).__name__))
@@ -54,8 +55,8 @@ def load(path=None, skip_failures=True):
                             cls.__name__))
                 __adapters__[cls.NAME] = cls
         else:
-            log.debug("ignored %r", sub)
-    log.debug("%d adapters loaded", len(__adapters__))
+            LOG.debug("ignored %r", sub)
+    LOG.debug("%d adapters loaded", len(__adapters__))
 
 def names():
     return __adapters__.keys()
