@@ -7,14 +7,14 @@ from .adapter import Adapter, AdapterError
 from .iomanager import IOManager, ServerMap
 from .reporter import FilesystemReporter, FuzzManagerReporter, Report, Reporter, S3FuzzManagerReporter
 from .runner import Runner, RunResult
-from .status import ReducerStats, Status
+from .status import Status
 from .storage import TestCaseLoadFailure, TestCase, TestFile, TestFileExists
 from .utils import grz_tmp
 
 
 __all__ = (
     "Adapter", "AdapterError", "FilesystemReporter", "FuzzManagerReporter", "grz_tmp", "IOManager",
-    "ReducerStats", "Report", "Reporter", "Runner", "RunResult", "S3FuzzManagerReporter",
+    "Report", "Reporter", "Runner", "RunResult", "S3FuzzManagerReporter",
     "ServerMap", "Status", "TestCase", "TestCaseLoadFailure", "TestFile", "TestFileExists")
 __author__ = "Jesse Schwartzentruber"
 __credits__ = ["Jesse Schwartzentruber", "Tyson Smith"]
