@@ -6,7 +6,7 @@
 from abc import ABCMeta, abstractmethod
 from collections import namedtuple
 from hashlib import sha1
-from json import dump
+from json import dump, dumps, loads
 from logging import getLogger, WARNING
 from platform import machine, system
 from os import listdir, getcwd, getenv, makedirs, mkdir, SEEK_END, stat, unlink, walk
@@ -50,7 +50,7 @@ LOG = getLogger(__name__)
 LogMap = namedtuple("LogMap", "aux stderr stdout")
 
 
-class Report(object):
+class Report:
     DEFAULT_MAJOR = "NO_STACK"
     DEFAULT_MINOR = "0"
     MAX_LOG_SIZE = 1048576  # 1MB
@@ -385,15 +385,16 @@ class Reporter(metaclass=ABCMeta):
             report (Report): Report to submit.
 
         Returns:
-            None
+            *: implementation specific result indicating where the report was created
         """
         assert isinstance(report, Report)
         assert report.path is not None
         self._pre_submit(report)
-        self._submit_report(report, test_cases)
+        result = self._submit_report(report, test_cases)
         if report is not None:
             report.cleanup()
         self._post_submit()
+        return result
 
 
 class FilesystemReporter(Reporter):
@@ -435,6 +436,7 @@ class FilesystemReporter(Reporter):
         free_space = disk_usage(log_path).free
         if free_space < self.DISK_SPACE_ABORT:
             raise RuntimeError("Running low on disk space (%0.1fMB)" % (free_space / 1048576.0,))
+        return dest_path
 
 
 class FuzzManagerReporter(Reporter):
@@ -445,7 +447,7 @@ class FuzzManagerReporter(Reporter):
     # testcase quality values
     QUAL_REDUCED_RESULT = 0  # the final reduced testcase
     QUAL_REDUCED_ORIGINAL = 1  # the original used for successful reduction
-    QUAL_REPRODUCIBLE = 4  # the testcase was reproducible
+    QUAL_REDUCING = 4  # the testcase is currently being reduced
     QUAL_UNREDUCED = 5  # haven't attempted reduction yet (1st attempt, generic reducer)
     QUAL_REQUEST_SPECIFIC = 6  # platform specific reduction requested (2nd attempt)
     QUAL_NO_TESTCASE = 7  # no testcase was detected (could be the "testcase" is not a testcase)
@@ -460,7 +462,7 @@ class FuzzManagerReporter(Reporter):
         self.tool = tool  # optional tool name
 
     def _post_submit(self):
-        self._extra_metadata = {}
+        self._extra_metadata.clear()
 
     @staticmethod
     def sanity_check(bin_file):
@@ -478,6 +480,22 @@ class FuzzManagerReporter(Reporter):
             raise IOError("Missing: %s.fuzzmanagerconf" % (bin_file,))
         ProgramConfiguration.fromBinary(bin_file)
 
+    def add_extra_metadata(self, key, value):
+        """Add extra metadata to be reported with any CrashEntrys reported.
+
+        Arguments:
+            key (str): key for this data in the metadata dict
+            value (object): JSON serializable object to be included in the FM crash metadata.
+                            The object will be deep-copied.
+
+        Returns:
+            None
+        """
+        assert isinstance(key, str)
+        assert key not in self._extra_metadata
+        # this not only does a deep copy, but also ensures that value is JSON serializable
+        self._extra_metadata[key] = loads(dumps(value))
+
     @classmethod
     def quality_name(cls, value):
         for name in dir(cls):
@@ -493,7 +511,7 @@ class FuzzManagerReporter(Reporter):
         trace_path = pathjoin(report.path, "rr-traces")
         if isdir(trace_path):
             LOG.info("Ignored rr trace")
-            self._extra_metadata["rr-trace"] = "ignored"
+            self.add_extra_metadata("rr-trace",  "ignored")
             # remove traces so they are not uploaded to FM (because they are huge)
             # use S3FuzzManagerReporter instead
             rmtree(trace_path)
@@ -527,7 +545,7 @@ class FuzzManagerReporter(Reporter):
                     LOG.info("Frequent crash matched existing signature: %s",
                              cache_metadata["shortDescription"])
                     if not self.force_report:
-                        return
+                        return None
                 elif "bug__id" in cache_metadata:
                     LOG.info("Crash matched existing signature (bug %s): %s",
                              cache_metadata["bug__id"],
@@ -548,7 +566,7 @@ class FuzzManagerReporter(Reporter):
             if cache_sig_file is None:
                 if self._ignored(report):
                     LOG.info("Report is unsupported and is in ignore list")
-                    return
+                    return None
                 LOG.warning("Report is unsupported by FM, saved to %r", report.path)
                 # TODO: we should check if stackhasher failed too
                 raise RuntimeError("Failed to create FM signature")
@@ -611,6 +629,8 @@ class FuzzManagerReporter(Reporter):
         if isfile(zip_name):
             unlink(zip_name)
 
+        return new_entry["id"]
+
 
 class S3FuzzManagerReporter(FuzzManagerReporter):
     @staticmethod
@@ -651,7 +671,7 @@ class S3FuzzManagerReporter(FuzzManagerReporter):
         else:
             # The object already exists.
             LOG.info("rr trace exists at %r", s3_url)
-            self._extra_metadata["rr-trace"] = s3_url
+            self.add_extra_metadata("rr-trace", s3_url)
             # remove traces so they are not reported to FM
             rmtree(trace_path)
             return s3_url
@@ -660,7 +680,7 @@ class S3FuzzManagerReporter(FuzzManagerReporter):
         rr_arc = self.compress_rr_trace(trace_path, report.path)
         s3.meta.client.upload_file(rr_arc, s3_bucket, s3_key, ExtraArgs={"ACL": "public-read"})
         unlink(rr_arc)
-        self._extra_metadata["rr-trace"] = s3_url
+        self.add_extra_metadata("rr-trace", s3_url)
         return s3_url
 
     @staticmethod
