@@ -2,6 +2,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # pylint: disable=protected-access
+from itertools import count
 from os.path import isfile
 from platform import system
 
@@ -19,7 +20,7 @@ def test_puppet_target_01(mocker, tmp_path):
     fake_ffp.return_value.log_length.return_value = 562
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000) as target:
         assert target.closed
         assert not target._remove_prefs
         prefs_file = target.prefs
@@ -37,7 +38,7 @@ def test_puppet_target_01(mocker, tmp_path):
     assert fake_ffp.return_value.clean_up.call_count == 1
     assert not isfile(prefs_file)
     # with extra args
-    with PuppetTarget(str(fake_file), None, 1, 1, 1, 1, rr=True, fake=1) as target:
+    with PuppetTarget(str(fake_file), None, 1, 1, 1, rr=True, fake=1) as target:
         pass
 
 def test_puppet_target_02(mocker, tmp_path):
@@ -46,7 +47,7 @@ def test_puppet_target_02(mocker, tmp_path):
     fake_file = tmp_path / "fake"
     fake_file.touch()
     # test providing prefs.js
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 35) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000) as target:
         target.prefs = str(fake_file)
         assert not target._remove_prefs
         # launch success
@@ -79,7 +80,7 @@ def test_puppet_target_03(mocker, tmp_path):
     fake_ffp.RC_WORKER = FFPuppet.RC_WORKER
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, 25)
+    target = PuppetTarget(str(fake_file), None, 300, 25, 5000)
     # no failures
     fake_ffp.return_value.is_healthy.return_value = True
     fake_ffp.return_value.reason = None
@@ -122,7 +123,11 @@ def test_puppet_target_03(mocker, tmp_path):
     fake_ffp.return_value.is_running.return_value = True
     fake_ffp.return_value.reason = None
     fake_ffp.return_value.cpu_usage.return_value = ((1234, 10), (1236, 75), (1238, 60))
-    fake_kill = mocker.patch("grizzly.target.puppet_target.kill", autospec=True)
+    # raise OSError for code coverage
+    fake_kill = mocker.patch(
+        "grizzly.target.puppet_target.kill",
+        autospec=True,
+        side_effect=OSError)
     assert target.detect_failure([], True) == Target.RESULT_FAILURE
     if system() == "Linux":
         assert fake_kill.call_count == 1
@@ -161,16 +166,6 @@ def test_puppet_target_03(mocker, tmp_path):
     fake_ffp.return_value.available_logs.return_value = " ffp_worker_log_size "
     assert target.detect_failure(["log-limit"], False) == Target.RESULT_IGNORED
     assert fake_ffp.return_value.close.call_count == 1
-    fake_ffp.reset_mock()
-    # test browser closing test case
-    fake_ffp.return_value.is_healthy.return_value = False
-    fake_ffp.return_value.is_running.return_value = False
-    fake_ffp.return_value.reason = FFPuppet.RC_EXITED
-    target.forced_close = False
-    target.rl_countdown = 0
-    assert target.detect_failure([], False) == Target.RESULT_NONE
-    assert fake_ffp.return_value.wait.call_count == 1
-    assert fake_ffp.return_value.close.call_count == 1
 
 @mark.skipif(system() == "Windows", reason="Unsupported on Windows")
 def test_puppet_target_04(mocker, tmp_path):
@@ -183,10 +178,9 @@ def test_puppet_target_04(mocker, tmp_path):
     fake_time = mocker.patch("grizzly.target.puppet_target.time", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    target = PuppetTarget(str(fake_file), None, 300, 25, 5000, 10)
+    target = PuppetTarget(str(fake_file), None, 300, 25, 5000)
     fake_kill = mocker.patch("grizzly.target.puppet_target.kill", autospec=True)
     # not running
-    target.rl_countdown = 1
     fake_ffp.return_value.get_pid.return_value = None
     target.dump_coverage()
     assert not fake_kill.call_count
@@ -250,6 +244,20 @@ def test_puppet_target_04(mocker, tmp_path):
     target.dump_coverage()
     assert fake_proc_iter.call_count == 3
     assert fake_kill.call_count == 2
+    fake_ffp.reset_mock()
+    fake_kill.reset_mock()
+    fake_proc_iter.reset_mock()
+    # kill calls raise OSError
+    fake_kill.side_effect = OSError
+    fake_ffp.return_value.is_healthy.return_value = True
+    fake_ffp.return_value.get_pid.return_value = 100
+    fake_proc_iter.side_effect = None
+    fake_time.side_effect = count()
+    target.dump_coverage()
+    assert fake_kill.call_count == 3
+    fake_ffp.reset_mock()
+    fake_kill.reset_mock()
+    fake_proc_iter.reset_mock()
 
 def test_puppet_target_05(mocker, tmp_path):
     """test is_idle()"""
@@ -257,7 +265,7 @@ def test_puppet_target_05(mocker, tmp_path):
     fake_ffp.return_value.cpu_usage.return_value = [(999, 30), (998, 20), (997, 10)]
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 10) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000) as target:
         assert not target.is_idle(0)
         assert not target.is_idle(25)
         assert target.is_idle(50)
@@ -267,7 +275,7 @@ def test_puppet_target_06(mocker, tmp_path):
     fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000) as target:
         fake_ffp.return_value.is_running.return_value = False
         fake_ffp.return_value.is_healthy.return_value = False
         assert target.monitor is not None
@@ -289,7 +297,7 @@ def test_puppet_target_07(mocker, tmp_path):
     mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     fake_file = tmp_path / "fake"
     fake_file.touch()
-    with PuppetTarget(str(fake_file), None, 300, 25, 5000, 25) as target:
+    with PuppetTarget(str(fake_file), None, 300, 25, 5000) as target:
         # default temp prefs
         assert not target._remove_prefs
         prefs_file = target.prefs
