@@ -31,7 +31,7 @@ SERVED_TIMEOUT = 3  # timeout occurred
 Tracker = namedtuple("Tracker", "files lock")
 
 
-class Job(object):
+class Job:
     # MIME_MAP is used to support new or uncommon mime types.
     # Definitions in here take priority over mimetypes.guess_type().
     MIME_MAP = {
@@ -88,6 +88,10 @@ class Job(object):
                     "required" if resource.required else "optional",
                     redirect,
                     resource.target)
+            for dyn_resp, resource in self.server_map.dynamic.items():
+                if resource.required:
+                    self._pending.files.add(dyn_resp)
+                    LOG.debug("%s: %r -> %r", "required", dyn_resp, resource.target)
         self.initial_queue_size = len(self._pending.files)
         LOG.debug("%d files required to serve", self.initial_queue_size)
 
