@@ -7,6 +7,7 @@ Sapphire unit tests
 import hashlib
 import os
 import random
+import socket
 import threading
 
 import pytest
@@ -17,7 +18,7 @@ from .worker import Worker
 from .server_map import ServerMap
 
 
-class _TestFile(object):
+class _TestFile:
     def __init__(self, url):
         self.code = None
         self.content_type = None
@@ -384,17 +385,13 @@ def test_sapphire_18(client, tmp_path):
     assert inc403.code == 403
 
 def test_sapphire_19(client, tmp_path):
-    """test dynamic response"""
-    _test_string = b"dynamic response -- TEST DATA!"
-
-    def _dyn_test_cb():
-        return _test_string
-
+    """test dynamic response - not required"""
+    _data = b"dynamic response -- TEST DATA!"
+    test_dr = _TestFile("dyn_test")
+    test_dr.len_org = len(_data)
+    test_dr.md5_org = hashlib.md5(_data).hexdigest()
     smap = ServerMap()
-    test_dr = _TestFile("dynm_test")
-    test_dr.len_org = len(_test_string)
-    test_dr.md5_org = hashlib.md5(_test_string).hexdigest()
-    smap.set_dynamic_response("dynm_test", _dyn_test_cb, mime_type="text/plain")
+    smap.set_dynamic_response("dyn_test", lambda: _data, mime_type="text/plain")
     test = _create_test("test_case.html", tmp_path)
     with Sapphire(timeout=10) as serv:
         client.launch("127.0.0.1", serv.port, [test_dr, test], in_order=True)
@@ -406,7 +403,23 @@ def test_sapphire_19(client, tmp_path):
     assert test_dr.len_srv == test_dr.len_org
     assert test_dr.md5_srv == test_dr.md5_org
 
-def test_sapphire_20(client_factory, tmp_path):
+def test_sapphire_20(client, tmp_path):
+    """test dynamic response - required"""
+    _data = b"dynamic response -- TEST DATA!"
+    test_dr = _TestFile("dyn_test")
+    test_dr.len_org = len(_data)
+    test_dr.md5_org = hashlib.md5(_data).hexdigest()
+    smap = ServerMap()
+    smap.set_dynamic_response("dyn_test", lambda: _data, mime_type="text/plain", required=True)
+    with Sapphire(timeout=10) as serv:
+        client.launch("127.0.0.1", serv.port, [test_dr], in_order=True)
+        assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
+    assert client.wait(timeout=10)
+    assert test_dr.code == 200
+    assert test_dr.len_srv == test_dr.len_org
+    assert test_dr.md5_srv == test_dr.md5_org
+
+def test_sapphire_21(client_factory, tmp_path):
     """test pending_files == 0 in worker thread"""
     client_defer = client_factory(rx_size=2)
     # server should shutdown while this file is being served
@@ -425,7 +438,7 @@ def test_sapphire_20(client_factory, tmp_path):
     assert test.code == 200
     assert test_defer.code == 0
 
-def test_sapphire_21(client, tmp_path):
+def test_sapphire_22(client, tmp_path):
     """test handling an invalid request"""
     bad_test = _TestFile("bad.html")
     bad_test.custom_request = b"a bad request...0+%\xef\xb7\xba\r\n"
@@ -438,7 +451,7 @@ def test_sapphire_21(client, tmp_path):
     assert test.code == 200
     assert bad_test.code == 400
 
-def test_sapphire_22(client, tmp_path):
+def test_sapphire_23(client, tmp_path):
     """test handling an empty request"""
     bad_test = _TestFile("bad.html")
     bad_test.custom_request = b""
@@ -451,7 +464,7 @@ def test_sapphire_22(client, tmp_path):
     assert test.code == 200
     assert bad_test.code == 0
 
-def test_sapphire_23(client_factory, tmp_path):
+def test_sapphire_24(client_factory, tmp_path):
     """test requesting multiple files via multiple connections"""
     to_serve = list()
     for i in range(2):
@@ -477,7 +490,7 @@ def test_sapphire_23(client_factory, tmp_path):
         assert t_file.code == 200
         assert t_file.len_srv == t_file.len_org
 
-def test_sapphire_24(client_factory, tmp_path):
+def test_sapphire_25(client_factory, tmp_path):
     """test all request types via multiple connections"""
     def _dyn_test_cb():
         return b"A" if random.getrandbits(1) else b"AA"
@@ -508,7 +521,7 @@ def test_sapphire_24(client_factory, tmp_path):
             clients[-1].launch("127.0.0.1", serv.port, to_serve, throttle=throttle)
         assert serv.serve_path(str(tmp_path), server_map=smap)[0] == SERVED_ALL
 
-def test_sapphire_25(client, tmp_path):
+def test_sapphire_26(client, tmp_path):
     """test dynamic response with bad callbacks"""
     test_dr = _TestFile("dynm_test")
     smap = ServerMap()
@@ -519,7 +532,7 @@ def test_sapphire_25(client, tmp_path):
         with pytest.raises(TypeError):
             serv.serve_path(str(tmp_path), server_map=smap)
 
-def test_sapphire_26(client, tmp_path):
+def test_sapphire_27(client, tmp_path):
     """test serving to a slow client"""
     t_data = "".join(random.choice("ABCD1234") for _ in range(0x19000))  # 100KB
     t_file = _create_test("test_case.html", tmp_path, data=t_data.encode("ascii"), calc_hash=True)
@@ -535,7 +548,7 @@ def test_sapphire_26(client, tmp_path):
     assert t_file.len_srv == t_file.len_org
     assert t_file.md5_srv == t_file.md5_org
 
-def test_sapphire_27(client, tmp_path):
+def test_sapphire_28(client, tmp_path):
     """test timeout while requesting multiple files"""
     files_to_serve = list()
     t_data = "".join(random.choice("ABCD1234") for _ in range(1024)).encode("ascii")
@@ -548,7 +561,7 @@ def test_sapphire_27(client, tmp_path):
     assert status == SERVED_TIMEOUT
     assert len(files_served) < len(files_to_serve)
 
-def test_sapphire_28(client_factory, tmp_path):
+def test_sapphire_29(client_factory, tmp_path):
     """test Sapphire.serve_path() with forever=True"""
     clients = list()
     with Sapphire(timeout=10) as serv:
@@ -569,7 +582,7 @@ def test_sapphire_28(client_factory, tmp_path):
     assert test.code == 200
     assert test.len_srv == test.len_org
 
-def test_sapphire_29(client, tmp_path):
+def test_sapphire_30(client, tmp_path):
     """test interesting file names"""
     to_serve = [
         # space in file name
@@ -582,7 +595,7 @@ def test_sapphire_29(client, tmp_path):
     assert client.wait(timeout=10)
     assert all(t_file.code == 200 for t_file in to_serve)
 
-def test_sapphire_30(client, tmp_path):
+def test_sapphire_31(client, tmp_path):
     """test interesting path string"""
     all_bytes = "".join(chr(i) for i in range(256))
     to_serve = [
@@ -596,7 +609,7 @@ def test_sapphire_30(client, tmp_path):
     assert client.wait(timeout=10)
     assert all(t_file.code is not None for t_file in to_serve)
 
-def test_sapphire_31(mocker):
+def test_sapphire_32(mocker):
     """test Sapphire._create_listening_socket()"""
     fake_sleep = mocker.patch("sapphire.core.sleep", autospec=True)
     fake_sock = mocker.patch("sapphire.core.socket", autospec=True)
@@ -624,6 +637,19 @@ def test_sapphire_31(mocker):
     assert fake_sock.return_value.listen.call_count == 1
     assert fake_sleep.call_count == 1
 
+def test_sapphire_33(mocker):
+    """test Sapphire.clear_backlog()"""
+    mocker.patch("sapphire.core.socket", autospec=True)
+    mocker.patch("sapphire.core.time", autospec=True, return_value=1)
+    pending = mocker.Mock(spec=socket.socket)
+    with Sapphire(timeout=10) as serv:
+        serv._socket = mocker.Mock(spec=socket.socket)
+        serv._socket.accept.side_effect = ((pending, None), OSError, BlockingIOError)
+        serv.clear_backlog()
+        assert serv._socket.accept.call_count == 3
+        assert serv._socket.settimeout.call_count == 2
+    assert pending.close.call_count == 1
+
 def test_main_01(mocker, tmp_path):
     """test Sapphire.main()"""
     args = mocker.Mock(
